@@ -1,8 +1,6 @@
 /*
- * $Id: abox.c,v 1.25 2005/01/16 16:48:00 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -187,6 +185,7 @@ static void abox_init(GTypeInstance *object, gpointer gclass)
 				1, 2, i, i + 1,
 				GTK_SHRINK, GTK_SHRINK, 4, 1);
 		abox->cmp_name[i] = gtk_label_new("");
+		gtk_label_set_line_wrap(GTK_LABEL(abox->cmp_name[i]), TRUE);
 		gtk_misc_set_alignment(GTK_MISC(abox->cmp_name[i]), 0., 0.5);
 		gtk_table_attach(GTK_TABLE(abox->cmp_area),
 				abox->cmp_name[i],
@@ -680,6 +679,10 @@ void    abox_set_percentage(ABox *abox, int per)
 				abox->progress, FALSE, FALSE, 2);
 		gtk_widget_show(abox->progress);
 	}
+	if(per<0 || per>100) {
+		gtk_widget_hide(abox->progress);
+		return;
+	}
 	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(abox->progress),
 				      per/100.);
 }
