@@ -1,8 +1,6 @@
 /*
- * $Id: collection.c,v 1.116 2005/07/31 19:04:53 tal197 Exp $
- *
- * Collection - a GTK+ widget
- * Copyright (C) 2005, the ROX-Filer team.
+ * ROX-Filer, filer for the ROX desktop project
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * The collection widget provides an area for displaying a collection of
  * objects (such as files). It allows the user to choose a selection of
@@ -543,9 +541,13 @@ static gint collection_expose(GtkWidget *widget, GdkEventExpose *event)
 	g_return_val_if_fail(IS_COLLECTION(widget), FALSE);
 	g_return_val_if_fail(event != NULL, FALSE);
 
+	/* Note about 'detail' argument:
+	 * - If set to "base", lighthouse theme will crash
+	 * - If set to NULL, cleanice theme will crash
+	 */
 	gtk_paint_flat_box(widget->style, widget->window, GTK_STATE_NORMAL, 
 		      GTK_SHADOW_NONE, &event->area,
-		      widget, "base", 0, 0, -1, -1);
+		      widget, "collection", 0, 0, -1, -1);
 
 	collection = COLLECTION(widget);
 
