@@ -1,8 +1,6 @@
 /*
- * $Id: gui_support.c,v 1.136 2005/10/02 16:48:30 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -447,6 +445,14 @@ gboolean get_pointer_xy(int *x, int *y)
 	return mask != 0;
 }
 
+int get_monitor_under_pointer(void)
+{
+	int x, y;
+
+	get_pointer_xy(&x, &y);
+	return gdk_screen_get_monitor_at_point(gdk_screen_get_default(), x, y);
+}
+
 #define DECOR_BORDER 32
 
 /* Centre the window at these coords */
@@ -1425,10 +1431,11 @@ void launch_uri(const char *uri)
 			argv[0] = uri_0launch;
 		else
 		{
-			delayed_error(_("This program cannot be run, as the "
-				"0launch command is not available. "
+			delayed_error(_("This program (%s) cannot be run, "
+				"as the 0launch command is not available. "
 				"It can be downloaded from here:\n\n"
-				"http://0install.net/injector.html"));
+				"http://0install.net/injector.html"),
+				uri);
 			return;
 		}
 	}
@@ -1500,3 +1507,67 @@ gint current_event_button(void)
 
 	return button;
 }
+
+/* Create a new pixbuf by colourizing 'src' to 'color'. If the function fails,
+ * 'src' will be returned (with an increased reference count, so it is safe to
+ * g_object_unref() the return value whether the function fails or not).
+ */
+GdkPixbuf *create_spotlight_pixbuf(GdkPixbuf *src, GdkColor *color)
+{
+	guchar opacity = 192;
+	guchar alpha = 255 - opacity;
+	GdkPixbuf *dst;
+	GdkColorspace colorspace;
+	int width, height, src_rowstride, dst_rowstride, x, y;
+	int n_channels, bps;
+	int r, g, b;
+	guchar *spixels, *dpixels, *src_pixels, *dst_pixels;
+	gboolean has_alpha;
+
+	has_alpha = gdk_pixbuf_get_has_alpha(src);
+	colorspace = gdk_pixbuf_get_colorspace(src);
+	n_channels = gdk_pixbuf_get_n_channels(src);
+	bps = gdk_pixbuf_get_bits_per_sample(src);
+
+	if ((colorspace != GDK_COLORSPACE_RGB) ||
+	    (!has_alpha && n_channels != 3) ||
+	    (has_alpha && n_channels != 4) ||
+	    (bps != 8))
+		goto error;
+
+	width = gdk_pixbuf_get_width(src);
+	height = gdk_pixbuf_get_height(src);
+
+	dst = gdk_pixbuf_new(colorspace, has_alpha, bps, width, height);
+	if (dst == NULL)
+		goto error;
+
+	src_pixels = gdk_pixbuf_get_pixels(src);
+	dst_pixels = gdk_pixbuf_get_pixels(dst);
+	src_rowstride = gdk_pixbuf_get_rowstride(src);
+	dst_rowstride = gdk_pixbuf_get_rowstride(dst);
+
+	r = opacity * (color->red >> 8);
+	g = opacity * (color->green >> 8);
+	b = opacity * (color->blue >> 8);
+
+	for (y = 0; y < height; y++)
+	{
+		spixels = src_pixels + y * src_rowstride;
+		dpixels = dst_pixels + y * dst_rowstride;
+		for (x = 0; x < width; x++)
+		{
+			*dpixels++ = (*spixels++ * alpha + r) >> 8;
+			*dpixels++ = (*spixels++ * alpha + g) >> 8;
+			*dpixels++ = (*spixels++ * alpha + b) >> 8;
+			if (has_alpha)
+				*dpixels++ = *spixels++;
+		}
+
+	}
+	return dst;
+
+error:
+	g_object_ref(src);
+	return src;
+}
