@@ -1,8 +1,6 @@
 /*
- * $Id: icon.c,v 1.84 2005/01/16 16:48:01 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -54,7 +52,7 @@
 #include "mount.h"
 #include "type.h"
 #include "usericons.h"
-#include "pinboard.h"	/* For pinboard_set_backdrop */
+#include "pinboard.h"	/* For pinboard_set_backdrop_box */
 
 static gboolean have_primary = FALSE;	/* We own the PRIMARY selection? */
 
@@ -62,6 +60,7 @@ GtkWidget		*icon_menu;		/* The popup icon menu */
 static GtkWidget	*icon_file_menu;	/* The file submenu */
 static GtkWidget	*icon_file_item;	/* 'File' label */
 static GtkWidget	*file_shift_item;	/* 'Shift Open' label */
+static GtkWidget	*current_options_item;	/* Pin/Pan Options */
 
 /* A list of selected Icons. Every icon in the list is from the same group
  * (eg, you can't have icons from two different panels selected at the
@@ -103,7 +102,6 @@ static void selection_get(GtkClipboard *primary,
 		       guint      info,
 		       gpointer   data);
 static void remove_items(gpointer data, guint action, GtkWidget *widget);
-static void set_backdrop(gpointer data, guint action, GtkWidget *widget);
 static void file_op(gpointer data, guint action, GtkWidget *widget);
 static void show_rename_box(Icon *icon);
 static void icon_set_selected_int(Icon *icon, gboolean selected);
@@ -111,7 +109,6 @@ static void icon_class_init(gpointer gclass, gpointer data);
 static void icon_init(GTypeInstance *object, gpointer gclass);
 static void icon_hash_path(Icon *icon);
 static void icon_unhash_path(Icon *icon);
-static void menu_set_hidden(GtkWidget *menu, gboolean hidden, int from, int n);
 static void ungrab_key(Icon *icon);
 static void grab_key(Icon *icon);
 static void parseKeyString(MyKey *key, const char *str);
@@ -147,7 +144,6 @@ static GtkItemFactoryEntry menu_def[] = {
 {N_("Show Location"),  		NULL, file_op, ACTION_LOCATION, "<StockItem>", GTK_STOCK_JUMP_TO},
 {N_("Remove Item(s)"),		NULL, remove_items, 0, "<StockItem>", GTK_STOCK_REMOVE},
 {"",				NULL, NULL, 0, "<Separator>"},
-{N_("Backdrop..."),		NULL, set_backdrop, 0, NULL},
 };
 
 /****************************************************************
@@ -173,7 +169,7 @@ void icon_may_update(Icon *icon)
 	if (di_image(icon->item) != image || icon->item->flags != flags)
 	{
 		/* Appearance changed; need to redraw */
-		g_signal_emit_by_name(icon, "update");
+		g_signal_emit_by_name(icon, "redraw");
 	}
 
 	if (image)
@@ -229,25 +225,61 @@ gboolean icons_require(const gchar *path)
 	return check.found;
 }
 
+static gboolean any_selected_item_is_locked()
+{
+	GList *next;
+
+	for (next = icon_selection; next; next = next->next)
+	{
+		Icon *icon = (Icon *)next->data;
+
+		if (icon->locked)
+			return TRUE;
+	}
+
+	return FALSE;
+}
+
 /* Menu was clicked over this icon. Set things up correctly (shade items,
  * add app menu stuff, etc).
  * You should show icon_menu after calling this...
+ * panel_name is NULL for the pinboard.
  */
-void icon_prepare_menu(Icon *icon, gboolean pinboard)
+void icon_prepare_menu(Icon *icon, GtkWidget *options_item)
 {
+	GtkWidget *image;
+	gboolean shaded;
+
 	appmenu_remove();
 
+	if (current_options_item)
+	{
+		gtk_widget_destroy(current_options_item);
+		current_options_item = NULL;
+	}
+
 	menu_icon = icon;
 
 	if (!icon_menu)
 		create_menu();
 
-	menu_set_hidden(icon_menu, !pinboard, 5, 2);
+	current_options_item = options_item;
+	image = gtk_image_new_from_stock(GTK_STOCK_PREFERENCES,
+						GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image(GTK_IMAGE_MENU_ITEM(options_item), image);
 
-	/* Shade Remove Item(s) unless there is a selection */
-	menu_set_items_shaded(icon_menu,
-		(icon_selection || menu_icon) ? FALSE : TRUE,
-		4, 1);
+	gtk_menu_shell_append(GTK_MENU_SHELL(icon_menu), options_item);
+	gtk_widget_show_all(options_item);
+	
+	/* Shade Remove Item(s) if any item is locked or nothing is selected */
+	if (icon_selection)
+		shaded = any_selected_item_is_locked();
+	else if (menu_icon)
+		shaded = menu_icon->locked;
+	else
+		shaded = TRUE;
+	
+	menu_set_items_shaded(icon_menu, shaded, 4, 1);
 
 	menu_show_shift_action(file_shift_item, icon ? icon->item : NULL,
 			FALSE);
@@ -328,7 +360,8 @@ void icon_destroy(Icon *icon)
 
 	icon_set_selected_int(icon, FALSE);
 
-	g_signal_emit_by_name(icon, "destroy");
+	if (!icon->locked)
+		g_signal_emit_by_name(icon, "destroy");
 }
 
 /* Return a text/uri-list of all the icons in the selection */
@@ -506,21 +539,24 @@ static void icon_unhash_path(Icon *icon)
 
 static void rename_activate(GtkWidget *dialog)
 {
-	GtkWidget *entry, *src, *shortcut, *arg;
+	GtkWidget *entry, *src, *shortcut, *arg, *lock_state;
 	Icon *icon;
 	const guchar *new_name, *new_src, *new_shortcut, *new_args;
+	gboolean new_lock_state;
 	
 	entry = g_object_get_data(G_OBJECT(dialog), "new_name");
 	icon = g_object_get_data(G_OBJECT(dialog), "callback_icon");
 	src = g_object_get_data(G_OBJECT(dialog), "new_path");
 	shortcut = g_object_get_data(G_OBJECT(dialog), "new_shortcut");
 	arg = g_object_get_data(G_OBJECT(dialog), "new_arg");
+	lock_state = g_object_get_data(G_OBJECT(dialog), "new_lock_state");
 
 	g_return_if_fail(entry != NULL &&
 			 src != NULL &&
 			 icon != NULL &&
 			 shortcut != NULL &&
-			 arg != NULL);
+			 arg != NULL &&
+			 lock_state != NULL);
 
 	new_name = gtk_entry_get_text(GTK_ENTRY(entry));
 	new_src = gtk_entry_get_text(GTK_ENTRY(src));
@@ -528,7 +564,8 @@ static void rename_activate(GtkWidget *dialog)
 	if (strcmp(new_shortcut, CLICK_TO_SET) == 0)
 		new_shortcut = NULL;
 	new_args = gtk_entry_get_text(GTK_ENTRY(arg));
-	
+	new_lock_state = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(lock_state));
+
 	if (*new_src == '\0')
 		report_error(
 			_("The location must contain at least one character!"));
@@ -537,6 +574,7 @@ static void rename_activate(GtkWidget *dialog)
 		icon_set_path(icon, new_src, new_name);
 		icon_set_shortcut(icon, new_shortcut);
 		icon_set_arguments(icon, new_args);
+		icon->locked = new_lock_state;
 		g_signal_emit_by_name(icon, "update");
 		gtk_widget_destroy(dialog);
 	}
@@ -587,17 +625,20 @@ static void selection_get(GtkClipboard	*primary,
 	gtk_selection_data_set_text(selection_data, text, strlen(text));
 }
 
-static void set_backdrop(gpointer data, guint action, GtkWidget *widget)
-{
-	pinboard_set_backdrop_box();
-}
-
 static void remove_items(gpointer data, guint action, GtkWidget *widget)
 {
 	IconClass	*iclass;
 
 	if (menu_icon)
+	{
+		if (menu_icon->locked)
+		{
+			delayed_error(_("You must unlock '%s' before removing it"),
+					menu_icon->item->leafname);
+			return;
+		}
 		icon_set_selected(menu_icon, TRUE);
+	}
 	
 	if (!icon_selection)
 	{
@@ -606,6 +647,12 @@ static void remove_items(gpointer data, guint action, GtkWidget *widget)
 		return;
 	}
 
+	if (any_selected_item_is_locked())
+	{
+		delayed_error(_("An item must be unlocked before it can be removed."));
+		return;
+	}
+
 	iclass = (IconClass *)
 		  G_OBJECT_GET_CLASS(G_OBJECT(icon_selection->data));
 
@@ -775,7 +822,7 @@ static void clear_shortcut(GtkButton *clear, GtkLabel *label)
 static void show_rename_box(Icon *icon)
 {
 	GtkDialog	*dialog;
-	GtkWidget	*label, *entry, *button, *button2, *hbox, *spacer;
+	GtkWidget	*label, *entry, *button, *button2, *hbox, *spacer, *lock_state;
 	GtkBox		*vbox;
 
 	if (icon->dialog)
@@ -785,6 +832,7 @@ static void show_rename_box(Icon *icon)
 	}
 
 	icon->dialog = gtk_dialog_new();
+	gtk_dialog_set_has_separator(GTK_DIALOG(icon->dialog), FALSE);
 	g_signal_connect(icon->dialog, "destroy",
 			G_CALLBACK(gtk_widget_destroyed), &icon->dialog);
 
@@ -854,6 +902,14 @@ static void show_rename_box(Icon *icon)
 	g_signal_connect(button2, "clicked",
 			G_CALLBACK(clear_shortcut),
 			GTK_BIN(button)->child);
+			
+	lock_state = gtk_check_button_new_with_label(_("Locked"));
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(lock_state), icon->locked);
+	gtk_box_pack_start(vbox, lock_state, TRUE, TRUE, 0);
+	g_object_set_data(G_OBJECT(dialog), "new_lock_state", lock_state);
+	gtk_tooltips_set_tip(tooltips, lock_state,
+			_("Locking an item prevents it from being accidentally removed"),
+			NULL);
 
 	g_object_set_data(G_OBJECT(dialog), "callback_icon", icon);
 
@@ -943,6 +999,7 @@ static void icon_init(GTypeInstance *object, gpointer gclass)
 	icon->shortcut_key.keycode = 0;
 	icon->shortcut_key.modifier = 0;
 	icon->args = NULL;
+	icon->locked = FALSE;
 }
 
 /* As icon_set_selected(), but doesn't automatically unselect incompatible
@@ -994,25 +1051,7 @@ static void icon_set_selected_int(Icon *icon, gboolean selected)
 	g_signal_emit_by_name(icon, "redraw");
 }
 
-static void menu_set_hidden(GtkWidget *menu, gboolean hidden, int from, int n)
-{
-	GList	*items, *item;
-
-	items = gtk_container_get_children(GTK_CONTAINER(menu));
-
-	item = g_list_nth(items, from);
-	while (item && n--)
-	{
-		if (hidden)
-			gtk_widget_hide(GTK_WIDGET(item->data));
-		else
-			gtk_widget_show(GTK_WIDGET(item->data));
-		item = item->next;
-	}
-	g_list_free(items);
-}
-
-/* Stolen from xfwm4 */
+/* From xfwm4 */
 static void initModifiers(void)
 {
 	static gboolean need_init = TRUE;
