@@ -1,8 +1,6 @@
 /*
- * $Id: infobox.c,v 1.68 2005/01/18 18:17:04 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -82,6 +80,7 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window);
 static GtkWidget *make_details(const guchar *path, DirItem *item,
 				GObject *window);
 static GtkWidget *make_about(const guchar *path, XMLwrapper *ai);
+static GtkWidget *make_about_desktop(const gchar *path);
 static GtkWidget *make_file_says(const guchar *path);
 static GtkWidget *make_permissions(const gchar *path, DirItem *item);
 static void add_file_output(FileStatus *fs,
@@ -240,27 +239,10 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window)
 		g_free(u8);
 	}
 	gtk_label_set_selectable(GTK_LABEL(name), TRUE);
+	gtk_label_set_line_wrap(GTK_LABEL(name), TRUE);
 	gtk_box_pack_start(GTK_BOX(hbox), name, FALSE, TRUE, 4);
 	
-	/* Make the name bolder and larger */
-	{
-		PangoAttribute *attr;
-		PangoAttrList *list;
-
-		list = pango_attr_list_new();
-
-		attr = pango_attr_weight_new(PANGO_WEIGHT_BOLD);
-		attr->start_index = 0;
-		attr->end_index = -1;
-		pango_attr_list_insert(list, attr);
-
-		attr = pango_attr_scale_new(PANGO_SCALE_X_LARGE);
-		attr->start_index = 0;
-		attr->end_index = -1;
-		pango_attr_list_insert(list, attr);
-		
-		gtk_label_set_attributes(GTK_LABEL(name), list);
-	}
+	make_heading(name, PANGO_SCALE_X_LARGE);
 
 	/* List of file attributes */
 	add_frame(vbox, make_details(path, item, window));
@@ -297,6 +279,10 @@ static GtkWidget *make_vbox(const guchar *path, GObject *window)
 
 	if (about)
 		add_frame(vbox, make_about(path, ai));
+	else if (item->mime_type == application_x_desktop)
+	{
+		add_frame(vbox, make_about_desktop(path));
+	}
 	else if (item->base_type == TYPE_FILE)
 	{
 		label = gtk_label_new(NULL);
@@ -519,20 +505,24 @@ static void cell_edited(GtkCellRendererText *cell,
 
 	oldlink = readlink_dup(fullpath);
 	if (!oldlink) {
-		report_error(_("'%s' is no longer a symlink"), fullpath);
+		/* Must use delayed_error(), as this can be called
+		 * from a focus-out event (causes a crash).
+		 */
+		delayed_error(_("'%s' is no longer a symlink"), fullpath);
 		return;
 	}
 	if (strcmp(oldlink, new_text) == 0)
 		return;	/* No change */
 	g_free(oldlink);
 	if (unlink(fullpath)) {
-		report_error(_("Failed to unlink '%s':\n%s"),
+		delayed_error(_("Failed to unlink '%s':\n%s"),
 				fullpath, g_strerror(errno));
 		return;
 	}
 	if (symlink(new_text, fullpath)) {
-		report_error(_("Failed to create symlink from '%s':\n%s\n"
-				"(note: old link has been deleted)"));
+		delayed_error(_("Failed to create symlink from '%s':\n%s\n"
+				"(note: old link has been deleted)"),
+				fullpath, g_strerror(errno));
 		return;
 	}
 
@@ -606,6 +596,10 @@ static GtkWidget *make_details(const guchar *path, DirItem *item,
 						     NULL))
 			{
 				du->chan = g_io_channel_unix_new(out);
+				/* Select binary encoding so we don't get an
+				 * error with non-UTF-8 filenames.
+				 */
+				g_io_channel_set_encoding(du->chan, NULL, NULL);
 				du->watch = g_io_add_watch(du->chan,
 						   G_IO_IN|G_IO_ERR|G_IO_HUP,
 						 (GIOFunc) read_du_output, du);
@@ -635,11 +629,11 @@ static GtkWidget *make_details(const guchar *path, DirItem *item,
 	if (item->mime_type)
 		add_row(store, "", mime_type_comment(item->mime_type));
 
-	if (xtype_supported(NULL)) {
+	if (xattr_supported(NULL)) {
 		add_row(store, _("Extended attributes:"),
 		  (item->flags & ITEM_FLAG_HAS_XATTR)
 		  	? _("Present")
-			: xtype_supported(path) ? _("None")
+			: xattr_supported(path) ? _("None")
 						: _("Not supported"));
 	}
 
@@ -662,8 +656,15 @@ static GtkWidget *make_details(const guchar *path, DirItem *item,
 	}
 
 	if (item->base_type != TYPE_DIRECTORY)
-		add_row_and_free(store, _("Run action:"),
-				 describe_current_command(item->mime_type));
+	{
+		if (EXECUTABLE_FILE(item))
+			add_row(store, _("Run action:"), _("Execute file"));
+		else
+		{
+			add_row_and_free(store, _("Run action:"),
+				describe_current_command(item->mime_type));
+		}
+	}
 
 	return view;
 }
@@ -743,6 +744,39 @@ static GtkWidget *make_about(const guchar *path, XMLwrapper *ai)
 	return view;
 }
 
+/* Create the TreeView widget with the desktop entry's details */
+static GtkWidget *make_about_desktop(const gchar *path)
+{
+	GtkListStore	*store;
+	GtkWidget	*view;
+	GError          *error=NULL;
+	gchar           *name=NULL, *comment=NULL, *exec=NULL;
+
+	make_list(&store, &view, NULL);
+
+	if(!get_values_from_desktop_file(path, &error,
+					 "Desktop Entry", "Name", &name,
+					 "Desktop Entry", "Comment", &comment,
+					 "Desktop Entry", "Exec", &exec,
+					 NULL))
+	{
+		/* Report it? */
+		delayed_error("%s", error->message);
+		if(error)
+			g_error_free(error);
+		return view;
+	}
+
+	if(name)
+		add_row_and_free(store, _("Name"), name);
+	if(comment)
+		add_row_and_free(store, _("Comment"), comment);
+	if(exec)
+		add_row_and_free(store, _("Execute"), exec);
+	
+	return view;
+}
+
 static GtkWidget *make_file_says(const guchar *path)
 {
 	GtkWidget	*w_file_label;
