@@ -1,8 +1,6 @@
 /*
- * $Id: main.c,v 1.196 2005/11/21 20:05:55 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -133,7 +131,7 @@ GtkTooltips *tooltips = NULL;
        "  -u, --user		show user name in each window \n"	\
        "  -v, --version		display the version information and exit\n"   \
        "  -x, --examine=FILE	FILE has changed - re-examine it\n"	\
-       "\nReport bugs to " BUGS_TO ".\n"		\
+       "\nReport bugs to %s.\n"		\
        "Home page (including updated versions): http://rox.sourceforge.net/\n")
 
 #define SHORT_OPS "c:d:t:b:l:r:B:op:s:hvnux:m:D:RS"
@@ -400,7 +398,7 @@ int main(int argc, char **argv)
 				show_features();
 				return EXIT_SUCCESS;
 			case 'h':
-				g_print("%s", _(HELP));
+			        g_print(_(HELP), BUGS_TO);
 				g_print("%s", _(SHORT_ONLY_WARNING));
 				return EXIT_SUCCESS;
 			case 'D':
@@ -817,6 +815,8 @@ static GtkWidget *launch_button_new(const char *label, const char *uri)
 {
 	GtkWidget *button;
 	GClosure *closure;
+	const gchar *slash;
+	gchar *tip;
 
 	button = button_new_mixed(GTK_STOCK_PREFERENCES, label);
 	closure = g_cclosure_new_swap(G_CALLBACK(launch_uri),
@@ -826,6 +826,18 @@ static GtkWidget *launch_button_new(const char *label, const char *uri)
 
 	allow_right_click(button);
 
+	slash = strrchr(uri, '/');
+	if (!slash)
+		slash = uri - 1;
+	tip = g_strdup_printf(
+			_("Left-click to run %s.\n"
+			  "Right-click for a list of versions."),
+			slash + 1);
+
+	gtk_tooltips_set_tip(tooltips, button, tip, NULL);
+
+	g_free(tip);
+
 	return button;
 }
 
