@@ -1,8 +1,6 @@
 /*
- * $Id: options.c,v 1.95 2005/10/02 16:48:30 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005 Thomas Leonard
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -1183,7 +1181,10 @@ static void update_cb(gpointer key, gpointer value, gpointer data)
 	Option *option = (Option *) value;
 
 	g_return_if_fail(option != NULL);
-	g_return_if_fail(option->widget != NULL);
+	if (option->widget == NULL) {
+		g_warning("No widget for option %s", (char *) key);
+		return;
+	}
 
 	updating_widgets++;
 	
