@@ -10,6 +10,8 @@ extern Pinboard	*current_pinboard;
 
 extern Icon *pinboard_drag_in_progress;
 
+extern Option o_pinboard_tasklist_per_workspace;
+
 extern PangoFontDescription *pinboard_font;
 extern GdkColor pin_text_fg_col, pin_text_bg_col;
 
@@ -23,12 +25,11 @@ void pinboard_init(void);
 void pinboard_activate(const gchar *name);
 void pinboard_pin(const gchar *path, const gchar *name, int x, int y,
 		  const gchar *shortcut);
-void pinboard_pin_with_arg(const gchar *path, const gchar *name, int x, int y,
-			   const gchar *shortcut, const char *arg);
+void pinboard_pin_with_args(const gchar *path, const gchar *name, int x, int y,
+			   const gchar *shortcut, const char *arg, gboolean locked);
 gboolean pinboard_remove(const gchar *path, const gchar *name);
 void pinboard_move_icons(void);
 const gchar *pinboard_get_name(void);
-void pinboard_set_backdrop_box(void);
 void pinboard_set_backdrop_app(const gchar *app);
 GdkWindow *pinboard_get_window(void);
 void pinboard_add_widget(GtkWidget *widget, const gchar *name);
