@@ -1,8 +1,6 @@
 /*
- * $Id: remote.c,v 1.66 2005/12/11 15:04:27 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -51,6 +49,7 @@
 #include "display.h"
 #include "xml.h"
 #include "diritem.h"
+#include "usericons.h"
 
 static GdkAtom filer_atom;	/* _ROX_FILER_EUID_VERSION_HOST */
 static GdkAtom filer_atom_any;	/* _ROX_FILER_EUID_HOST */
@@ -102,6 +101,9 @@ static xmlNodePtr rpc_PinboardRemove(GList *args);
 static xmlNodePtr rpc_SetBackdrop(GList *args);
 static xmlNodePtr rpc_SetBackdropApp(GList *args);
 
+static xmlNodePtr rpc_SetIcon(GList *args);
+static xmlNodePtr rpc_UnsetIcon(GList *args);
+
 /****************************************************************
  *			EXTERNAL INTERFACE			*
  ****************************************************************/
@@ -144,10 +146,12 @@ gboolean remote_init(xmlDocPtr rpc, gboolean new_copy)
 
 	soap_register("SetBackdrop", rpc_SetBackdrop, "Filename,Style", NULL);
 	soap_register("SetBackdropApp", rpc_SetBackdropApp, "App", NULL);
-	soap_register("PinboardAdd", rpc_PinboardAdd, "Path,X,Y", "Label,Shortcut,Args");
+	soap_register("PinboardAdd", rpc_PinboardAdd, "Path,X,Y", "Label,Shortcut,Args,Locked");
 	soap_register("PinboardRemove", rpc_PinboardRemove, "Path", "Label");
-	soap_register("PanelAdd", rpc_PanelAdd, "Side,Path", "Label,After,Shortcut,Args");
+	soap_register("PanelAdd", rpc_PanelAdd, "Side,Path", "Label,After,Shortcut,Args,Locked");
 	soap_register("PanelRemove", rpc_PanelRemove, "Side", "Path,Label");
+ 	soap_register("SetIcon", rpc_SetIcon, "Path,Icon", NULL);
+ 	soap_register("UnsetIcon", rpc_UnsetIcon, "Path", NULL);
 
 	/* Look for a property on the root window giving the IPC window
 	 * of an already-running copy of this version of the filer, running
@@ -567,6 +571,34 @@ static GList *list_value(xmlNode *arg)
  * NULL if there isn't a result.
  */
 
+static xmlNodePtr rpc_SetIcon(GList *args)
+{
+	char	   *path, *icon;
+	
+	path = string_value(ARG(0));
+	icon = string_value(ARG(1));
+
+	add_globicon(path,icon);
+
+	g_free(path);
+	g_free(icon);
+
+	return NULL;
+}
+
+static xmlNodePtr rpc_UnsetIcon(GList *args)
+{
+	char	   *path;
+	
+	path = string_value(ARG(0));
+
+	delete_globicon(path);
+
+	g_free(path);
+
+	return NULL;
+}
+
 static xmlNodePtr rpc_Version(GList *args)
 {
 	xmlNodePtr reply;
@@ -795,12 +827,12 @@ static xmlNodePtr rpc_SetBackdropApp(GList *args)
 	return NULL;
 }
 
-/* args = Path, X, Y, [Label] */
+/* args = Path, X, Y, [Label, Shortcut, Args, Locked] */
 static xmlNodePtr rpc_PinboardAdd(GList *args)
 {
 	char *path = NULL;
 	gchar *name, *shortcut, *xargs;
-	int x, y;
+	int x, y, locked;
 
 	path = string_value(ARG(0));
 	x = int_value(ARG(1), 0);
@@ -808,8 +840,10 @@ static xmlNodePtr rpc_PinboardAdd(GList *args)
 	name = string_value(ARG(3));
 	shortcut = string_value(ARG(4));
 	xargs = string_value(ARG(5));
+	locked = bool_value(ARG(6));
 
-	pinboard_pin_with_args(path, name, x, y, shortcut, xargs);
+	pinboard_pin_with_args(path, name, x, y, shortcut, xargs, 
+						(locked==-1) ? FALSE : locked);
 
 	g_free(path);
 	g_free(name);
@@ -862,12 +896,13 @@ static xmlNodePtr rpc_Panel(GList *args)
 	return NULL;
 }
 
-/* args = Side, Path, [Label, After] */
+/* args = Side, Path, [Label, After, Shortcut, Args, Locked] */
 static xmlNodePtr rpc_PanelAdd(GList *args)
 {
 	PanelSide side;
 	char *path, *side_name, *label, *shortcut, *arg;
 	gboolean after = FALSE;
+	gboolean locked = FALSE;
 	int tmp;
 
 	side_name = string_value(ARG(0));
@@ -883,8 +918,9 @@ static xmlNodePtr rpc_PanelAdd(GList *args)
 
 	shortcut = string_value(ARG(4));
 	arg = string_value(ARG(5));
+	locked = bool_value(ARG(6));
 
-	panel_add(side, path, label, after, shortcut, arg);
+	panel_add(side, path, label, after, shortcut, arg, (locked== -1) ? FALSE : locked);
 
 	g_free(path);
 	g_free(label);
