@@ -1,8 +1,6 @@
 /*
- * $Id: support.c,v 1.116 2005/01/24 12:06:21 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -25,6 +23,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <stdarg.h>
 #include <netdb.h>
 #include <errno.h>
 #include <ctype.h>
@@ -38,6 +37,7 @@
 #include <unistd.h>
 #include <libxml/parser.h>
 #include <math.h>
+#include <sys/mman.h>
 
 #include "global.h"
 
@@ -1413,3 +1413,248 @@ int stat_with_timeout(const char *path, struct stat *info)
 
 	return retval;
 }
+
+/* From glib. */
+static gchar *my_strchrnul(const gchar *str, gchar c)
+{
+	gchar *p = (gchar*) str;
+	while (*p && (*p != c))
+		++p;
+
+	return p;
+}
+
+/* Based on code from glib. */
+gboolean available_in_path(const char *file)
+{
+	const gchar *path, *p;
+	gchar *name, *freeme;
+	size_t len;
+	size_t pathlen;
+	gboolean found = FALSE;
+
+	path = g_getenv("PATH");
+	if (path == NULL)
+		path = "/bin:/usr/bin:.";
+
+	len = strlen(file) + 1;
+	pathlen = strlen(path);
+	freeme = name = g_malloc(pathlen + len + 1);
+
+	/* Copy the file name at the top, including '\0'  */
+	memcpy(name + pathlen + 1, file, len);
+	name = name + pathlen;
+	/* And add the slash before the filename  */
+	*name = '/';
+
+	p = path;
+	do
+	{
+		char *startp;
+
+		path = p;
+		p = my_strchrnul(path, ':');
+
+		if (p == path)
+			/* Two adjacent colons, or a colon at the beginning or
+			 * the end of `PATH' means to search the current
+			 * directory.
+			 */
+			startp = name + 1;
+		else
+			startp = memcpy (name - (p - path), path, p - path);
+
+		/* Try to execute this name.  If it works, execv will not
+		 * return.
+		 */
+		if (access(startp, X_OK) == 0)
+			found = TRUE;
+	} while (!found && *p++ != '\0');
+
+	g_free(freeme);
+
+	return found;
+}
+
+static char *get_value_from_desktop_data(const char *data, size_t length,
+					 const char *section, const char *key,
+					 GError **error)
+{
+	int section_length, key_length;
+	const char *last_possible_key_start;
+	const char *last_possible_section_start;
+	const char *next;
+
+	section_length = strlen(section);
+	last_possible_section_start = data + length - section_length - 3;
+
+	for (next = data; next <= last_possible_section_start; next++)
+	{
+		if (next[0] != '[' || (next > data && next[-1] != '\n'))
+			continue;
+		next += 1;
+		if (next[section_length] != ']' ||
+		    next[section_length + 1] != '\n')
+			continue;
+		if (strncmp(next, section, section_length) != 0)
+			continue;
+		next += section_length + 1;	/* Newline */
+		goto find_key;
+	}
+	return NULL;
+
+find_key:
+	key_length = strlen(key);
+	last_possible_key_start = data + length - key_length - 3;
+	for (;
+	     next && next < last_possible_key_start;
+	     next = memchr(next + 1, '\n', last_possible_key_start - next))
+	{
+		const char *nl;
+
+		if (next[1] == '\n')
+			continue; /* Cope with blank lines */
+
+		next++;		/* Skip newline */
+
+		if (next[0] == '[')
+			break;	/* End of section */
+
+		if (strncmp(next, key, key_length) != 0)
+			continue;
+		next += key_length;
+		while (next[0] == ' ' || next[0] == '\t')
+			next++;
+
+		/* Note: if we had GLib 2.6, we could use
+		 * g_get_language_names() to get translations here.
+		 */
+		if (next[0] != '=')
+			continue;
+		next++;
+		while (next[0] == ' ' || next[0] == '\t')
+			next++;
+		nl = memchr(next, '\n', data + length - next);
+		if (!nl)
+			break;
+		return g_strndup(next, nl - next);
+	}
+
+	return NULL;
+}
+
+/* Load .desktop file 'path' and return the value of the named key.
+ * Sets error if the desktop file cannot be parsed.
+ * Returns NULL (but does not set error) if the key is not present.
+ * Returned string must be g_free()d.
+ */
+char *get_value_from_desktop_file(const char *path,
+				  const char *section,
+				  const char *key,
+				  GError **error)
+{
+	char *value = NULL;
+	struct stat info;
+	int fd = -1;
+	void *start = NULL;
+
+	fd = open(path, O_RDONLY);
+	if (fd == -1 || fstat(fd, &info) != 0)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to open and stat file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+	start = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0);
+	if (start == MAP_FAILED)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to mmap file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+
+	value = get_value_from_desktop_data((const char *) start,
+					    info.st_size,
+					    section, key,
+					    error);
+err:
+	if (fd != -1)
+		close(fd);
+
+	if (start != NULL && start != MAP_FAILED)
+		munmap(start, info.st_size);
+
+	return value;
+}
+
+/* Load .desktop file 'path' and set the value of the named keys in the
+ * NULL terminated list.
+ * Sets error if the desktop file cannot be parsed and returns NULL.
+ * Sets NULL (but does not set error) if the key is not present.
+ * String set must be g_free()d.
+ */
+gboolean get_values_from_desktop_file(const char *path,
+				      GError **error,
+				      const char *section,
+				      const char *key,
+				      gchar **value, ...)
+{
+	struct stat info;
+	int fd = -1;
+	void *start = NULL;
+	va_list list;
+
+	fd = open(path, O_RDONLY);
+	if (fd == -1 || fstat(fd, &info) != 0)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to open and stat file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+	start = mmap(NULL, info.st_size, PROT_READ, MAP_SHARED, fd, 0);
+	if (start == MAP_FAILED)
+	{
+                g_set_error(error,
+                             G_FILE_ERROR,
+                             g_file_error_from_errno(errno),
+                             _("Failed to mmap file '%s': %s"),
+                             path,
+                             g_strerror(errno));
+		goto err;
+	}
+
+	va_start(list, value);
+	while(section && key && value) {
+		*value = get_value_from_desktop_data((const char *) start,
+						     info.st_size,
+						     section, key,
+						     error);
+		if(*error)
+			break;
+
+		section=va_arg(list, const char *);
+		key=va_arg(list, const char *);
+		value=va_arg(list, char **);
+	}
+		
+err:
+	if (fd != -1)
+		close(fd);
+
+	if (start != NULL && start != MAP_FAILED)
+		munmap(start, info.st_size);
+
+	return (*error==NULL);
+}
