@@ -1,8 +1,6 @@
 /*
- * $Id: toolbar.c,v 1.82 2005/08/01 07:10:20 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -148,7 +146,8 @@ static Tool all_tools[] = {
 	 toolbar_sort_clicked, DROP_NONE, FALSE,
 	 FALSE},
 	
-	{N_("Hidden"), ROX_STOCK_SHOW_HIDDEN, N_("Show/hide hidden files"),
+	{N_("Hidden"), ROX_STOCK_SHOW_HIDDEN, N_("Left: Show/hide hidden files\n"
+						 "Right: Show/hide thumbnails"),
 	 toolbar_hidden_clicked, DROP_NONE, TRUE,
 	 FALSE},
 	
@@ -184,6 +183,8 @@ void toolbar_update_info(FilerWindow *filer_window)
 	ViewIface	*view;
 	int		n_selected;
 
+	g_return_if_fail(filer_window != NULL);
+
 	if (o_toolbar.int_value == TOOLBAR_NONE || !o_toolbar_info.int_value)
 		return;		/* Not showing info */
 
@@ -472,7 +473,18 @@ static void toolbar_details_clicked(GtkWidget *widget,
 static void toolbar_hidden_clicked(GtkWidget *widget,
 				   FilerWindow *filer_window)
 {
-	display_set_hidden(filer_window, !filer_window->show_hidden);
+	GdkEvent	*event;
+
+	event = gtk_get_current_event();
+	if (event->type == GDK_BUTTON_RELEASE &&
+			((GdkEventButton *) event)->button == 1)
+	{
+		display_set_hidden(filer_window, !filer_window->show_hidden);
+	}
+	else
+	{
+		display_set_thumbs(filer_window, !filer_window->show_thumbs);
+	}
 }
 
 static gboolean invert_cb(ViewIter *iter, gpointer data)
@@ -733,10 +745,11 @@ static void handle_drops(FilerWindow *filer_window,
 
 static void tally_items(gpointer key, gpointer value, gpointer data)
 {
-	guchar *leafname = (guchar *) key;
-	int    *tally = (int *) data;
+	gchar  *leafname = (gchar *) key;
+	DirItem *item = (DirItem *) value;
+	int     *tally = (int *) data;
 
-	if (!filer_match_filter(filer_window_being_counted, leafname))
+	if (!filer_match_filter(filer_window_being_counted, item))
 		(*tally)++;
 }
 
