@@ -1,8 +1,6 @@
 /*
- * $Id: type.c,v 1.174 2005/11/29 20:40:58 tal197 Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -60,6 +58,7 @@
 #include "dropbox.h"
 #include "xdgmime.h"
 #include "xtypes.h"
+#include "run.h"
 
 #define TYPE_NS "http://www.freedesktop.org/standards/shared-mime-info"
 enum {SET_MEDIA, SET_TYPE};
@@ -309,10 +308,11 @@ MIME_type *type_from_path(const char *path)
 /* Returns the file/dir in Choices for handling this type.
  * NULL if there isn't one. g_free() the result.
  */
-static char *handler_for(MIME_type *type)
+char *handler_for(MIME_type *type)
 {
 	char	*type_name;
 	char	*open;
+	char	*target;
 
 	type_name = g_strconcat(type->media_type, "_", type->subtype, NULL);
 	open = choices_find_xdg_path_load(type_name, "MIME-types", SITE);
@@ -322,7 +322,39 @@ static char *handler_for(MIME_type *type)
 		open = choices_find_xdg_path_load(type->media_type,
 						  "MIME-types", SITE);
 
-	return open;
+	if (!open)
+		return NULL;
+
+	/* Some programs behave differently depending on the command
+	 * name (e.g., 'vim' vs 'gvim'), so symlinks need to be followed here.
+	 */
+	target = readlink_dup(open);
+	if (!target)
+	{
+		return open;
+	}
+
+	if (target[0] == '/')
+	{
+		/* Absolute path */
+		g_free(open);
+		return target;
+	}
+	else
+	{
+		/* Relative path (shouldn't normally be needed) */
+		gchar *dir;
+		char *abs_path;
+
+		dir = g_path_get_dirname(open);
+		g_free(open);
+
+		abs_path = g_strconcat(dir, "/", target, NULL);
+		g_free(target);
+		g_free(dir);
+
+		return abs_path;
+	}
 }
 
 MIME_type *mime_type_lookup(const char *type)
@@ -332,66 +364,6 @@ MIME_type *mime_type_lookup(const char *type)
 
 /*			Actions for types 			*/
 
-gboolean type_open(const char *path, MIME_type *type)
-{
-	gchar *argv[] = {NULL, NULL, NULL};
-	char		*open;
-	gboolean	retval;
-	struct stat	info;
-
-	argv[1] = (char *) path;
-
-	open = handler_for(type);
-	if (!open)
-		return FALSE;
-
-	if (stat(open, &info))
-	{
-		report_error("stat(%s): %s", open, g_strerror(errno));
-		g_free(open);
-		return FALSE;
-	}
-
-	if (info.st_mode & S_IWOTH)
-	{
-		gchar *choices_dir;
-		GList *paths;
-
-		report_error(_("Executable '%s' is world-writeable! Refusing "
-			"to run. Please change the permissions now (this "
-			"problem may have been caused by a bug in earlier "
-			"versions of the filer).\n\n"
-			"Having (non-symlink) run actions world-writeable "
-			"means that other people who use your computer can "
-			"replace your run actions with malicious versions.\n\n"
-			"If you trust everyone who could write to these files "
-			"then you needn't worry. Otherwise, you should check, "
-			"or even just delete, all the existing run actions."),
-			open);
-		choices_dir = g_path_get_dirname(open);
-		paths = g_list_append(NULL, choices_dir);
-		action_chmod(paths, TRUE, _("go-w (Fix security problem)"));
-		g_free(choices_dir);
-		g_list_free(paths);
-		g_free(open);
-		return TRUE;
-	}
-
-	if (S_ISDIR(info.st_mode))
-		argv[0] = g_strconcat(open, "/AppRun", NULL);
-	else
-		argv[0] = open;
-
-	retval = rox_spawn(home_dir, (const gchar **) argv) != 0;
-
-	if (argv[0] != open)
-		g_free(argv[0]);
-
-	g_free(open);
-	
-	return retval;
-}
-
 /* Return the image for this type, loading it if needed.
  * Places to check are: (eg type="text_plain", base="text")
  * 1. <Choices>/MIME-icons/base_subtype
@@ -403,7 +375,7 @@ gboolean type_open(const char *path, MIME_type *type)
  */
 MaskedPixmap *type_to_icon(MIME_type *type)
 {
-	GdkPixbuf *full;
+	GtkIconInfo *full;
 	char	*type_name, *path;
 	time_t	now;
 
@@ -442,8 +414,7 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 
 	type_name = g_strconcat("mime-", type->media_type, ":",
 				type->subtype, NULL);
-	full = gtk_icon_theme_load_icon(icon_theme, type_name, HUGE_HEIGHT,
-						0, NULL);
+	full = gtk_icon_theme_lookup_icon(icon_theme, type_name, HUGE_HEIGHT, 0);
 	g_free(type_name);
 	if (!full)
 	{
@@ -453,18 +424,18 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 		else
 			type_name = g_strconcat("gnome-mime-", type->media_type,
 					"-", type->subtype, NULL);
-		full = gtk_icon_theme_load_icon(icon_theme,
+		full = gtk_icon_theme_lookup_icon(icon_theme,
 						type_name,
-						HUGE_HEIGHT, 0, NULL);
+						HUGE_HEIGHT, 0);
 		g_free(type_name);
 	}
 	if (!full)
 	{
 		/* Try for a media type */
 		type_name = g_strconcat("mime-", type->media_type, NULL);
-		full = gtk_icon_theme_load_icon(icon_theme,
+		full = gtk_icon_theme_lookup_icon(icon_theme,
 						type_name,
-						HUGE_HEIGHT, 0, NULL);
+						HUGE_HEIGHT, 0);
 		g_free(type_name);
 	}
 	if (!full)
@@ -472,15 +443,24 @@ MaskedPixmap *type_to_icon(MIME_type *type)
 		/* Ugly hack... try for a GNOME default media icon */
 		type_name = g_strconcat("gnome-mime-", type->media_type, NULL);
 
-		full = gtk_icon_theme_load_icon(icon_theme,
+		full = gtk_icon_theme_lookup_icon(icon_theme,
 				type_name,
-				HUGE_HEIGHT, 0, NULL);
+				HUGE_HEIGHT, 0);
 		g_free(type_name);
 	}
 	if (full)
 	{
-		type->image = masked_pixmap_new(full);
-		g_object_unref(full);
+		const char *icon_path;
+		/* Get the actual icon through our cache, not through GTK, because
+		 * GTK doesn't cache icons.
+		 */
+		icon_path = gtk_icon_info_get_filename(full);
+		if (icon_path != NULL)
+			type->image = g_fscache_lookup(pixmap_cache, icon_path);
+		/* else shouldn't happen, because we didn't use
+		 * GTK_ICON_LOOKUP_USE_BUILTIN.
+		 */
+		gtk_icon_info_free(full);
 	}
 
 out:
@@ -748,7 +728,6 @@ gchar *describe_current_command(MIME_type *type)
 	char *handler;
 	char *desc = NULL;
 	struct stat info;
-	char *target;
 
 	g_return_val_if_fail(type != NULL, NULL);
 
@@ -757,28 +736,6 @@ gchar *describe_current_command(MIME_type *type)
 	if (!handler)
 		return g_strdup(_("No run action defined"));
 
-	target = readlink_dup(handler);
-	if (target)
-	{
-		/* Cope with relative paths (shouldn't normally be needed) */
-
-		if (target[0] == '/')
-		{
-			g_free(handler);
-			handler = target;
-		}
-		else
-		{
-			gchar *dir;
-
-			dir = g_path_get_dirname(handler);
-			g_free(handler);
-			handler = g_strconcat(dir, "/", target, NULL);
-			g_free(target);
-			g_free(dir);
-		}
-	}
-
 	if (mc_stat(handler, &info) !=0 )
 	{
 		desc = g_strdup_printf(_("Error in handler %s: %s"), handler,
