@@ -1,8 +1,6 @@
 /*
- * $Id: view_collection.c,v 1.45 2005/07/29 22:06:46 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -329,6 +327,7 @@ static void draw_item(GtkWidget *widget,
 	ViewCollection	*view_collection = (ViewCollection *) user_data;
 	FilerWindow	*filer_window = view_collection->filer_window;
 	GtkStateType	selection_state;
+	GdkColor	*color;
 
 	g_return_if_fail(view != NULL);
 
@@ -336,6 +335,8 @@ static void draw_item(GtkWidget *widget,
 		selection_state = filer_window->selection_state;
 	else
 		selection_state = GTK_STATE_NORMAL;
+
+	color = &widget->style->bg[selection_state];
 	
 	fill_template(area, colitem, view_collection, &template);
 		
@@ -350,18 +351,18 @@ static void draw_item(GtkWidget *widget,
 			template.icon.height <= SMALL_HEIGHT)
 	{
 		draw_small_icon(widget->window, &template.icon,
-				item, view->image, selected);
+				item, view->image, selected, color);
 	}
 	else if (template.icon.width <= ICON_WIDTH &&
 			template.icon.height <= ICON_HEIGHT)
 	{
 		draw_large_icon(widget->window, &template.icon,
-				item, view->image, selected);
+				item, view->image, selected, color);
 	}
 	else
 	{
 		draw_huge_icon(widget->window, &template.icon,
-				item, view->image, selected);
+				item, view->image, selected, color);
 	}
 	
 	draw_string(widget, view->layout,
@@ -1062,9 +1063,8 @@ static void view_collection_add_items(ViewIface *view, GPtrArray *items)
 	for (i = 0; i < items->len; i++)
 	{
 		DirItem *item = (DirItem *) items->pdata[i];
-		char	*leafname = item->leafname;
 
-		if (!filer_match_filter(filer_window, leafname))
+		if (!filer_match_filter(filer_window, item))
 			continue;
 
 		add_item(view_collection, item);
@@ -1095,7 +1095,7 @@ static void view_collection_update_items(ViewIface *view, GPtrArray *items)
 		const gchar *leafname = item->leafname;
 		int j;
 
-		if (!filer_match_filter(filer_window, leafname))
+		if (!filer_match_filter(filer_window, item))
 			continue;
 
 		j = collection_find_item(collection, item,
