@@ -1,8 +1,6 @@
 /*
- * $Id: view_details.c,v 1.46 2005/03/23 19:16:55 kerofin Exp $
- *
  * ROX-Filer, filer for the ROX desktop project
- * Copyright (C) 2005, the ROX-Filer team.
+ * Copyright (C) 2006, Thomas Leonard and others (see changelog for details).
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -927,6 +925,25 @@ static void view_details_class_init(gpointer gclass, gpointer data)
 	widget->expose_event = view_details_expose;
 	widget->size_request = view_details_size_request;
 	widget->drag_data_received = view_details_drag_data_received;
+
+	/*
+	 * Add the ViewDetails::mono-font style property.
+	 * To use this add something like
+	 *
+	 * style "details" {
+	 *   ViewDetails::mono-font = "Courier 8"
+	 * }
+	 * class "ViewDetails" style "details"
+	 *
+	 * to your ~/.gtkrc-2.0
+	 */
+	gtk_widget_class_install_style_property(widget,
+		       g_param_spec_string("mono-font",
+					   _("Mono font"),
+					   _("Font for displaying mono-spaced text"),
+					   "monospace",
+					   G_PARAM_READABLE));
+								    
 }
 
 static gboolean block_focus(GtkWidget *button, GtkDirectionType dir,
@@ -958,6 +975,21 @@ static void selection_changed(GtkTreeSelection *selection,
 			gtk_get_current_event_time());
 }
 
+/*
+ * Set the font used for a treeview column to that given for the
+ * "mono-font" style property of a widget.  This has to be done _after_ the
+ * widget has been realized, because that is when the style information is
+ * set up.  This is done by connecting this function to run after the
+ * "realize" signal.
+ */
+static void set_column_mono_font(GtkWidget *widget, GObject *object)
+{
+	const gchar *font_name;
+
+	gtk_widget_style_get(widget, "mono-font", &font_name, NULL);
+	g_object_set(object, "font", font_name, NULL);
+}
+
 #define ADD_TEXT_COLUMN(name, model_column) \
 	cell = gtk_cell_renderer_text_new();	\
 	column = gtk_tree_view_column_new_with_attributes(name, cell, \
@@ -990,14 +1022,17 @@ static void view_details_init(GTypeInstance *object, gpointer gclass)
 				GTK_SELECTION_MULTIPLE);
 	gtk_tree_selection_set_select_function(view_details->selection,
 			test_can_change_selection, view_details, NULL);
-	g_signal_connect(view_details->selection, "changed",
-			G_CALLBACK(selection_changed), view_details);
 
 	/* Sorting */
 	view_details->sort_fn = NULL;
 	sortable_list = GTK_TREE_SORTABLE(object);
 
 	gtk_tree_view_set_model(treeview, GTK_TREE_MODEL(view_details));
+	/* Do this after set_model, because that can generate this
+	 * signal...
+	 */
+	g_signal_connect(view_details->selection, "changed",
+			G_CALLBACK(selection_changed), view_details);
 
 	/* Icon */
 	cell = cell_icon_new(view_details);
@@ -1013,11 +1048,19 @@ static void view_details_init(GTypeInstance *object, gpointer gclass)
 	gtk_tree_view_column_set_sort_column_id(column, COL_TYPE);
 	gtk_tree_view_column_set_resizable(column, TRUE);
 	ADD_TEXT_COLUMN(_("_Permissions"), COL_PERM);
+	g_object_set(G_OBJECT(cell), "font", "monospace", NULL);
+	g_signal_connect_after(object, "realize",
+			       G_CALLBACK(set_column_mono_font),
+			       G_OBJECT(cell));
 	ADD_TEXT_COLUMN(_("_Owner"), COL_OWNER);
 	gtk_tree_view_column_set_sort_column_id(column, COL_OWNER);
 	ADD_TEXT_COLUMN(_("_Group"), COL_GROUP);
 	gtk_tree_view_column_set_sort_column_id(column, COL_GROUP);
 	ADD_TEXT_COLUMN(_("_Size"), COL_SIZE);
+	g_object_set(G_OBJECT(cell), "xalign", 1.0, "font", "monospace", NULL);
+	g_signal_connect_after(object, "realize",
+			       G_CALLBACK(set_column_mono_font),
+			       G_OBJECT(cell));
 	gtk_tree_view_column_set_sort_column_id(column, COL_SIZE);
 	ADD_TEXT_COLUMN(_("Last _Modified"), COL_MTIME);
 	gtk_tree_view_column_set_sort_column_id(column, COL_MTIME);
@@ -1179,7 +1222,7 @@ static void view_details_add_items(ViewIface *view, GPtrArray *new_items)
 		char	*leafname = item->leafname;
 		ViewItem *vitem;
 
-		if(!filer_match_filter(filer_window, leafname))
+		if(!filer_match_filter(filer_window, item))
 			continue;
 		if (leafname[0] == '.')
 		{
@@ -1270,7 +1313,7 @@ static void view_details_update_items(ViewIface *view, GPtrArray *items)
 		const gchar *leafname = item->leafname;
 		int j;
 
-		if (!filer_match_filter(filer_window, leafname))
+		if (!filer_match_filter(filer_window, item))
 			continue;
 
 		j = details_find_item(view_details, item);
